////////////////////////////////////////
/**************************************/
/*                                    */
/* Michael Goldbuch; Jrgen Dffinger */
/*                                    */
/* Mikroprozessortechnik              */
/* Praktikum 3 / Aufgabe1             */
/* Version 1.0                        */
/* 2011-05-03                         */
/*                                    */
/**************************************/
////////////////////////////////////////



//	definitions for ATMega8
.include "m8def.inc"



/***************   Makros   ***************/

.macro RECEIVE

RECEIVE_1:

	//	Wait for data to be received
	sbis UCSRA, RXC
	rjmp RECEIVE_1
	
	//	Get and return received data from buffer
	in R16, UDR

	//	Inkrementieren
	inc R16

.endm



.macro TRANSMIT

TRANSMIT_1:

	; Put data (r16) into buffer, sends the data
	sbis UCSRA,UDRE
	rjmp TRANSMIT_1
	out UDR,R16

.endm


/***************   Interupts setzen   ***************/

// Interrupts
rjmp RESET ; Reset Handler
reti; rjmp EXT_INT0 ; IRQ0 Handler
reti; rjmp EXT_INT1 ; IRQ1 Handler
reti; rjmp TIM2_COMP ; Timer2 Compare Handler
reti; rjmp TIM2_OVF ; Timer2 Overflow Handler
reti; rjmp TIM1_CAPT ; Timer1 Capture Handler
reti; rjmp TIM1_COMPA ; Timer1 CompareA Handler
reti; rjmp TIM1_COMPB ; Timer1 CompareB Handler
reti; rjmp TIM1_OVF ; Timer1 Overflow Handler
reti; rjmp TIM0_OVF ; Timer0 Overflow Handler
reti; rjmp SPI_STC ; SPI Transfer Complete Handler
rjmp USART_RXC ; USART RX Complete Handler
reti; rjmp USART_UDRE ; UDR Empty Handler
rjmp USART_TXC ; USART TX Complete Handler
reti; rjmp ADC ; ADC Conversion Complete Handler
reti; rjmp EE_RDY ; EEPROM Ready Handler
reti; rjmp ANA_COMP ; Analog Comparator Handler
reti; rjmp TWSI ; Two-wire Serial Interface Handler
reti; rjmp SPM_RDY ; Store Program Memory Ready Handler



/***************   RESET   ***************/

RESET:

	// 	Stack initialisieren
	ldi R16,high(RAMEND)
	out sph,R16			
	ldi R16,low(RAMEND)	
	out spl,R16		

	// 	USART initialisieren

	; Set baud rate (2400 BAUD)
	clr R17
	ldi R16,0xD0
	out UBRRH, R17
	out UBRRL, R16

	; Enable receiver and transmitter and the interupts
	ldi r16, (1<<RXEN)|(1<<TXEN)|(1<<RXCIE)|(1<<TXCIE)
	out UCSRB,r16
	
	; Set frame format: 8data, 2stop bit
	ldi r16, (1<<URSEL)|(1<<USBS)|(3<<UCSZ0)
	out UCSRC,r16

	//	hintere LEDs initialisieren
	sbi DDRD,7
	sbi DDRC,0
	sbi DDRC,1
	cbi PORTD,7
	cbi PORTC,0
	cbi PORTC,1

	// global interupt enable
	sei

rjmp MAIN



/***************   Hauptprogramm   ***************/

MAIN:
	rjmp MAIN



/***************   USART_RXC   ***************/

USART_RXC:
	
	//	Resgister sichern
	push R17
	in R17,SREG
	push R17

	// global Interupt disable
	cli
	
	RECEIVE 	

	//	receive disable	and transmit enable
	in   R17,UCSRB
	andi R17,0b01101111
	ori  R17,0b01001000
	out  UCSRB,R17
	
	//	global interupt enable
	sei

	TRANSMIT

	//	Register wiederherstellen
	pop R17
	out SREG,R17
	pop R17
reti



/***************   USART_TXC   ***************/

USART_TXC:
	
	//	Register sichern
	push R17
	in R17,SREG
	push R17

	//	receive enable and transmit disable
	in   R17,UCSRB
	andi R17,0b10110111
	ori  R17,0b10010000
	out  UCSRB,R17

	//	Register wiederherstellen
	pop R17
	out SREG,R17
	pop R17

reti
